@@ -60,15 +60,19 @@ struct _GstMad {
   struct mad_header header;
   gboolean	 new_header;
   gboolean	 can_seek;
-  gint		 channels;
   guint		 framecount;
   gint		 vbr_average; /* average bitrate */
   guint64	 vbr_rate; /* average * framecount */
 
-  GstCaps	*metadata;
+  gboolean	 half;
+  gboolean	 ignore_crc;
 
-  /* caps */
-  gboolean	 caps_set;
+  GstCaps	*metadata;
+  GstCaps	*streaminfo;
+
+  /* negotiated format */
+  gint		 rate;
+  gint		 channels;
 };
 
 struct _GstMadClass {
@@ -95,14 +99,10 @@ enum {
 
 enum {
   ARG_0,
-  ARG_LAYER,
-  ARG_MODE,
-  ARG_EMPHASIS,
-  ARG_BITRATE,
-  ARG_SAMPLERATE,
-  ARG_CHANNELS,
-  ARG_AVERAGE_BITRATE,
+  ARG_HALF,
+  ARG_IGNORE_CRC,
   ARG_METADATA,
+  ARG_STREAMINFO
   /* FILL ME */
 };
 
@@ -130,7 +130,7 @@ GST_PAD_TEMPLATE_FACTORY (mad_sink_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "mad_sink",
-    "audio/mp3",
+    "audio/x-mp3",
     NULL
   )
 )
@@ -191,7 +191,7 @@ gst_mad_get_type (void)
 }
 
 #define GST_TYPE_MAD_LAYER (gst_mad_layer_get_type())
-static GType
+G_GNUC_UNUSED static GType
 gst_mad_layer_get_type(void) {
   static GType mad_layer_type = 0;
   static GEnumValue mad_layer[] = {
@@ -207,24 +207,8 @@ gst_mad_layer_get_type(void) {
   return mad_layer_type;
 }
 
-#define GST_TYPE_MAD_CHANNELS (gst_mad_channels_get_type())
-static GType
-gst_mad_channels_get_type(void) {
-  static GType mad_channels_type = 0;
-  static GEnumValue mad_channels[] = {
-    {0, "0", "Unknown"},
-    {1, "1", "Mono"},
-    {2, "2", "Stereo"},
-    { 0, NULL, NULL},
-  };
-  if (!mad_channels_type) {
-    mad_channels_type = g_enum_register_static("GstMadChannels", mad_channels);
-  }
-  return mad_channels_type;
-}
-
 #define GST_TYPE_MAD_MODE (gst_mad_mode_get_type())
-static GType
+G_GNUC_UNUSED static GType
 gst_mad_mode_get_type(void) {
   static GType mad_mode_type = 0;
   static GEnumValue mad_mode[] = {
@@ -242,7 +226,7 @@ gst_mad_mode_get_type(void) {
 }
 
 #define GST_TYPE_MAD_EMPHASIS (gst_mad_emphasis_get_type())
-static GType
+G_GNUC_UNUSED static GType
 gst_mad_emphasis_get_type(void) {
   static GType mad_emphasis_type = 0;
   static GEnumValue mad_emphasis[] = {
@@ -278,30 +262,18 @@ gst_mad_class_init (GstMadClass *klass)
   /* init properties */
   /* currently, string representations are used, we might want to change that */
   /* FIXME: descriptions need to be more technical, default values and ranges need to be selected right */
-  g_object_class_install_property (gobject_class, ARG_LAYER,
-    g_param_spec_enum ("layer", "Layer", "The audio MPEG Layer this stream is encoded in",
-			GST_TYPE_MAD_LAYER, 0, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_MODE,
-    g_param_spec_enum ("mode", "Mode", "The current mode of the channels",
-		       GST_TYPE_MAD_MODE, -1, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_EMPHASIS,
-    g_param_spec_enum ("emphasis", "Emphasis", "Emphasis",
-		       GST_TYPE_MAD_EMPHASIS, -1, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_BITRATE,
-    g_param_spec_int ("bitrate", "Bitrate", "current bitrate of the stream",
-                       0, G_MAXINT, 0, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_AVERAGE_BITRATE,
-    g_param_spec_int ("average-bitrate", "average bitrate", "average bitrate of the stream",
-                       0, G_MAXINT, 0, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_SAMPLERATE,
-    g_param_spec_int ("samplerate", "Samplerate", "current samplerate of the stream",
-                       0, G_MAXINT, 0, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_CHANNELS,
-    g_param_spec_enum ("channels", "Channels", "number of channels",
-                       GST_TYPE_MAD_CHANNELS, 0, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class, ARG_HALF,
+    g_param_spec_boolean ("half", "Half", "Generate PCM at 1/2 sample rate",
+		          FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_IGNORE_CRC,
+    g_param_spec_boolean ("ignore_crc", "Ignore CRC", "Ignore CRC errors",
+		          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_METADATA,
     g_param_spec_boxed ("metadata", "Metadata", "Metadata",
                         GST_TYPE_CAPS, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
+    g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
+                        GST_TYPE_CAPS, G_PARAM_READABLE));
 }
 
 static void
@@ -338,6 +310,11 @@ gst_mad_init (GstMad *mad)
   mad->restart = FALSE;
   mad->header.mode = -1;
   mad->header.emphasis = -1;
+  mad->metadata = NULL;
+  mad->streaminfo = NULL;
+
+  mad->half = FALSE;
+  mad->ignore_crc = FALSE;
 
   GST_FLAG_SET (mad, GST_ELEMENT_EVENT_AWARE);
 }
@@ -601,6 +578,13 @@ gst_mad_src_event (GstPad *pad, GstEvent *event)
   mad = GST_MAD (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK_SEGMENT:
+      gst_event_ref (event);
+      if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), event)) {
+        /* seek worked, we're done, loop will exit */
+        res = TRUE;
+      }
+      break;
     /* the all-formats seek logic */
     case GST_EVENT_SEEK:
     {
@@ -694,6 +678,12 @@ gst_mad_set_property (GObject *object, guint prop_id, const GValue *value, GPara
   mad = GST_MAD (object);
 
   switch (prop_id) {
+    case ARG_HALF:
+      mad->half = g_value_get_boolean (value);
+      break;
+    case ARG_IGNORE_CRC:
+      mad->ignore_crc = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -710,52 +700,73 @@ gst_mad_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
   mad = GST_MAD (object);
 
   switch (prop_id) {
-    case ARG_LAYER:
-      g_value_set_enum (value, mad->header.layer);
+    case ARG_HALF:
+      g_value_set_boolean (value, mad->half);
       break;
-    case ARG_MODE:
-      g_value_set_enum (value, mad->header.mode);
-      break;
-    case ARG_EMPHASIS:
-      g_value_set_enum (value, mad->header.emphasis);
-      break;
-    case ARG_BITRATE:
-      g_value_set_int (value, mad->header.bitrate);
-      break;
-    case ARG_AVERAGE_BITRATE:
-      g_value_set_int (value, mad->vbr_average);
-      break;
-    case ARG_SAMPLERATE:
-      g_value_set_int (value, mad->header.samplerate);
-      break;
-    case ARG_CHANNELS:
-      g_value_set_enum (value, mad->channels);
+    case ARG_IGNORE_CRC:
+      g_value_set_boolean (value, mad->ignore_crc);
       break;
     case ARG_METADATA:
       g_value_set_boxed (value, mad->metadata);
       break;
+    case ARG_STREAMINFO:
+      g_value_set_boxed (value, mad->streaminfo);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
+static GstCaps*
+gst_mad_get_streaminfo (GstMad *mad)
+{
+  GstCaps *caps;
+  GstProps *props;
+  GstPropsEntry *entry;
+  GEnumValue *value;
+  GEnumClass *klass;
+
+  props = gst_props_empty_new ();
+
+  entry = gst_props_entry_new ("layer", GST_PROPS_INT (mad->header.layer));
+  gst_props_add_entry (props, (GstPropsEntry *) entry);
+
+  klass = g_type_class_ref (GST_TYPE_MAD_MODE);
+  value = g_enum_get_value (klass,
+		            mad->header.mode);
+  entry = gst_props_entry_new ("mode", GST_PROPS_STRING (value->value_nick));
+  g_type_class_unref (klass);
+  gst_props_add_entry (props, (GstPropsEntry *) entry);
+
+  klass = g_type_class_ref (GST_TYPE_MAD_EMPHASIS);
+  value = g_enum_get_value (klass,
+		            mad->header.emphasis);
+  entry = gst_props_entry_new ("emphasis", GST_PROPS_STRING (value->value_nick));
+  g_type_class_unref (klass);
+  gst_props_add_entry (props, (GstPropsEntry *) entry);
+
+  caps = gst_caps_new ("mad_streaminfo",
+		       "application/x-gst-streaminfo",
+		       props);
+  return caps;
+}
+
 static void
 gst_mad_update_info (GstMad *mad)
 {
   gint abr = mad->vbr_average;
   struct mad_header *header = &mad->frame.header;
+  gboolean changed = FALSE;
 
 #define CHECK_HEADER(h1,str) 					\
 G_STMT_START{							\
   if (mad->header.h1 != header->h1 || mad->new_header) {	\
     mad->header.h1 = header->h1;				\
-    g_object_notify (G_OBJECT (mad), str);			\
+     changed = TRUE;						\
   };								\
 } G_STMT_END
   
-  g_object_freeze_notify (G_OBJECT (mad));
-
   /* update average bitrate */
   if (mad->new_header) {
     mad->framecount = 1;
@@ -766,22 +777,23 @@ G_STMT_START{							\
     mad->vbr_rate += header->bitrate;
   }
   mad->vbr_average = (gint) (mad->vbr_rate / mad->framecount);
-  if (abr != mad->vbr_average) {
-    g_object_notify (G_OBJECT (mad), "average_bitrate");
-  }
 
   CHECK_HEADER (layer, 	    "layer");
   CHECK_HEADER (mode, 	    "mode");
   CHECK_HEADER (emphasis,   "emphasis");
-  CHECK_HEADER (bitrate,    "bitrate");
-  CHECK_HEADER (samplerate, "samplerate");
-  if (mad->channels != MAD_NCHANNELS (header) || mad->new_header) {
-    mad->channels = MAD_NCHANNELS (header);
-    g_object_notify (G_OBJECT (mad), "channels");
+
+  if (header->bitrate != mad->header.bitrate || mad->new_header) {
+    mad->header.bitrate = header->bitrate;
   }
   mad->new_header = FALSE;
-  
-  g_object_thaw_notify (G_OBJECT (mad));
+
+  if (changed) {
+    if (mad->streaminfo) {
+      gst_caps_unref (mad->streaminfo);
+    }
+    mad->streaminfo = gst_mad_get_streaminfo (mad);
+    g_object_notify (G_OBJECT (mad), "streaminfo");
+  }
   
 #undef CHECK_HEADER
 }
@@ -938,9 +950,6 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 	gint n = GST_EVENT_DISCONT_OFFSET_LEN (event);
 	gint i;
 
-	if (!GST_PAD_IS_USABLE (mad->srcpad))
-          return;
-
 	for (i=0; i<n; i++) {
 	  if (gst_pad_handles_format (pad, GST_EVENT_DISCONT_OFFSET(event,i).format))
 	  {
@@ -963,8 +972,11 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 	    mad->base_time = time;
 
             gst_event_unref (event);
-	    discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
-            gst_pad_push (mad->srcpad, GST_BUFFER (discont));
+
+	    if (GST_PAD_IS_USABLE (mad->srcpad)) {
+	      discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
+              gst_pad_push (mad->srcpad, GST_BUFFER (discont));
+	    }
 	    break;
 	  }
 	}
@@ -1017,10 +1029,9 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
     /* if we have data we can try to proceed */
     while (mad->tempsize >= 0) {
       gint consumed;
-      guint nchannels, nsamples;
-      mad_fixed_t const *left_ch, *right_ch;
-      GstBuffer *outbuffer;
-      gint16 *outdata;
+      guint nchannels;
+      guint nsamples;
+      gint rate;
 
       mad_stream_buffer (&mad->stream, mad_input_buffer, mad->tempsize);
 
@@ -1050,7 +1061,9 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 
 	    data = mad->stream.this_frame;
 	  
-	    mad_stream_skip(&mad->stream, tagsize);
+	    /* mad has moved the pointer to the next frame over the start of the
+	     * id3 tags, so we need to flush one byte less n the tagsize */
+	    mad_stream_skip(&mad->stream, tagsize-1);
 
 	    tag = id3_tag_parse (data, tagsize);
 	    if (tag) {
@@ -1067,19 +1080,59 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 	goto next;
       }
 
-      mad_synth_frame (&mad->synth, &mad->frame);
-
       nchannels = MAD_NCHANNELS (&mad->frame.header);
-      nsamples  = mad->synth.pcm.length;
-      left_ch   = mad->synth.pcm.samples[0];
-      right_ch  = mad->synth.pcm.samples[1];
+      nsamples  = MAD_NSBSAMPLES (&mad->frame.header) * 
+         (mad->stream.options & MAD_OPTION_HALFSAMPLERATE ? 16 : 32);  
+
+#if MAD_VERSION_MINOR <= 12
+      rate = mad->header.sfreq;
+#else
+      rate = mad->frame.header.samplerate;
+#endif
 
       /* at this point we can accept seek events */
       mad->can_seek = TRUE;
 
       gst_mad_update_info (mad);
 
+      if (mad->channels != nchannels || mad->rate != rate) {
+        if (mad->stream.options & MAD_OPTION_HALFSAMPLERATE) 
+	  rate >>=1;
+
+	/* we set the caps even when the pad is not connected so they
+	 * can be gotten for streaminfo */
+        if (gst_pad_try_set_caps (mad->srcpad,
+  	    gst_caps_new (
+  	      "mad_src",
+              "audio/raw",
+              gst_props_new (
+    	        "format",   GST_PROPS_STRING ("int"),
+                "law",         GST_PROPS_INT (0),
+                "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
+                "signed",      GST_PROPS_BOOLEAN (TRUE),
+                "width",       GST_PROPS_INT (16),
+                "depth",       GST_PROPS_INT (16),
+                "rate",        GST_PROPS_INT (rate),
+                "channels",    GST_PROPS_INT (nchannels),
+                NULL))) <= 0) 
+	{
+	  gst_buffer_unref (buffer);
+          gst_element_error (GST_ELEMENT (mad), 
+			     "could not set caps on source pad, aborting...");
+	  return;
+        }
+	mad->channels = nchannels;
+	mad->rate = rate;
+      }
+
       if (GST_PAD_IS_CONNECTED (mad->srcpad)) {
+        GstBuffer *outbuffer;
+        gint16 *outdata;
+        mad_fixed_t const *left_ch, *right_ch;
+
+        mad_synth_frame (&mad->synth, &mad->frame);
+        left_ch   = mad->synth.pcm.samples[0];
+        right_ch  = mad->synth.pcm.samples[1];
 
         outbuffer = gst_buffer_new_and_alloc (nsamples * nchannels * 2);
         outdata = (gint16 *) GST_BUFFER_DATA (outbuffer);
@@ -1092,49 +1145,25 @@ gst_mad_chain (GstPad *pad, GstBuffer *buffer)
 			 mad->total_samples * GST_SECOND / mad->frame.header.samplerate;
         }
 
-        mad->total_samples += nsamples;
-
         /* output sample(s) in 16-bit signed native-endian PCM */
         if (nchannels == 1) {
-          while (nsamples--) {
+          gint count = nsamples;
+          while (count--) {
             *outdata++ = scale(*left_ch++) & 0xffff;
           }
 	}
 	else {
-          while (nsamples--) {
+          gint count = nsamples;
+          while (count--) {
             *outdata++ = scale(*left_ch++) & 0xffff;
             *outdata++ = scale(*right_ch++) & 0xffff;
           }
         }
-        if (mad->caps_set == FALSE) {
-          if (gst_pad_try_set_caps (mad->srcpad,
-  	      gst_caps_new (
-  	        "mad_src",
-                "audio/raw",
-                gst_props_new (
-    	          "format",   GST_PROPS_STRING ("int"),
-                  "law",         GST_PROPS_INT (0),
-                  "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
-                  "signed",      GST_PROPS_BOOLEAN (TRUE),
-                  "width",       GST_PROPS_INT (16),
-                  "depth",       GST_PROPS_INT (16),
-#if MAD_VERSION_MINOR <= 12
-                  "rate",        GST_PROPS_INT (mad->header.sfreq),
-#else
-                  "rate",        GST_PROPS_INT (mad->header.samplerate),
-#endif
-                  "channels",    GST_PROPS_INT (nchannels),
-                  NULL))) <= 0) {
-            gst_element_error (GST_ELEMENT (mad), "could not set caps on source pad, aborting...");
-          }
-          mad->caps_set = TRUE;
-        }
 
         gst_pad_push (mad->srcpad, outbuffer);
       }
-      else {
-        mad->total_samples += nsamples;
-      }
+
+      mad->total_samples += nsamples;
 
       /* we have a queued timestamp on the incomming buffer that we should
        * use for the next frame */
@@ -1176,16 +1205,29 @@ gst_mad_change_state (GstElement *element)
     case GST_STATE_NULL_TO_READY:
       break;
     case GST_STATE_READY_TO_PAUSED:
+    {
+      guint options = 0;
+
       mad_stream_init (&mad->stream);
       mad_frame_init (&mad->frame);
       mad_synth_init (&mad->synth);
       mad->tempsize = 0;
       mad->can_seek = FALSE;
       mad->total_samples = 0;
-      mad->caps_set = FALSE;
+      mad->rate = 0;
+      mad->channels = 0;
       mad->vbr_average = 0;
+      mad->base_time = 0;
+      mad->framestamp = 0;
+      mad->new_header = TRUE;
+      mad->framecount = 0;
+      mad->vbr_rate = 0;
       mad->frame.header.samplerate = 0;
+      if (mad->ignore_crc) options |= MAD_OPTION_IGNORECRC;
+      if (mad->half) options |= MAD_OPTION_HALFSAMPLERATE;
+      mad_stream_options (&mad->stream, options);
       break;
+    }
     case GST_STATE_PAUSED_TO_PLAYING:
       /* do something to get out of the chain function faster */
       break;
@@ -1196,6 +1238,8 @@ gst_mad_change_state (GstElement *element)
       mad_frame_finish (&mad->frame);
       mad_stream_finish (&mad->stream);
       mad->restart = TRUE;
+      mad->metadata = NULL;
+      mad->streaminfo = NULL;
       break;
     case GST_STATE_READY_TO_NULL:
       break;
