@@ -21,13 +21,10 @@
 #ifndef __MPEG_PARSE_H__
 #define __MPEG_PARSE_H__
 
-
-#include <config.h>
 #include <gst/gst.h>
 #include <gst/bytestream/bytestream.h>
 #include "gstmpegpacketize.h"
 
-
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -46,7 +43,7 @@ extern "C" {
 
 #define GST_MPEG_PARSE_IS_MPEG2(parse) (GST_MPEG_PACKETIZE_IS_MPEG2 (GST_MPEG_PARSE (parse)->packetize))
 
-#define MPEGTIME_TO_GSTTIME(time) (((time) * GST_SECOND) / 90000LL)
+#define MPEGTIME_TO_GSTTIME(time) (((time) * (GST_MSECOND/10)) / 9LL)
 
 typedef struct _GstMPEGParse GstMPEGParse;
 typedef struct _GstMPEGParseClass GstMPEGParseClass;
@@ -59,16 +56,23 @@ struct _GstMPEGParse {
   GstMPEGPacketize *packetize;
 
   /* pack header values */
-  guint32	 bit_rate;
+  guint32	 mux_rate;
   guint64	 current_scr;
-  guint64	 previous_scr;
+  guint64	 next_scr;
+  guint64	 bytes_since_scr;
+
+  gint64	 adjust;
 
   gboolean	 discont_pending;
   gboolean	 scr_pending;
+  gint		 max_discont;
 
   GstClock 	*provided_clock;
   GstClock 	*clock;
   gboolean	 sync;
+
+  GstCache	*cache;
+  gint		 cache_id;
 };
 
 struct _GstMPEGParseClass {
