@@ -192,6 +192,7 @@ gst_videosink_class_init (GstVideoSinkClass *klass)
   /* gobject_class->dispose = gst_videosink_dispose; */
   
   gstelement_class->change_state = gst_videosink_change_state;
+  gstelement_class->set_clock = gst_videosink_set_clock;
   
   /* plugins */
   klass->plugins = NULL;
@@ -215,7 +216,6 @@ gst_videosink_init (GstVideoSink *sink)
   sink->height = 0;
   sink->muted = FALSE;
   sink->clock = NULL;  
-  GST_ELEMENT (sink)->setclockfunc = gst_videosink_set_clock;
   GST_FLAG_SET(sink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET (sink, GST_ELEMENT_EVENT_AWARE);
     
@@ -420,7 +420,7 @@ gst_videosink_chain (GstPad *pad, GstBuffer *buf)
   if (sink->clock && time != -1) {
     GstClockReturn ret;
 
-    ret = gst_clock_wait (sink->clock, GST_BUFFER_TIMESTAMP (buf), NULL);
+    ret = gst_element_clock_wait (GST_ELEMENT (sink), sink->clock, GST_BUFFER_TIMESTAMP (buf), NULL);
 
     /* we are going to drop early buffers */
     if (ret == GST_CLOCK_EARLY) {
