@@ -48,9 +48,6 @@ enum {
 
 enum {
   ARG_0,
-  ARG_WIDTH,
-  ARG_HEIGHT,
-  ARG_FORMAT,
   ARG_XID,
   ARG_SCALE,
   ARG_FRAMES_DISPLAYED,
@@ -116,15 +113,6 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_WIDTH,
-    g_param_spec_int ("width", "Width", "The width of the image",
-                      G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HEIGHT,
-    g_param_spec_int ("height", "Height", "The height of the image",
-                      G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FORMAT,
-    g_param_spec_string ("format", "Format", "The format of the image",
-                         "", G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_XID,
     g_param_spec_int ("xid", "Xid", "The Xid of the window",
                       G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
@@ -170,6 +158,8 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
   gobject_class->dispose = gst_xvideosink_dispose;
   
   gstelement_class->change_state = gst_xvideosink_change_state;
+  gstelement_class->set_clock = gst_xvideosink_set_clock;
+
 }
 
 static void
@@ -369,12 +359,6 @@ gst_xvideosink_sinkconnect (GstPad *pad, GstCaps *caps)
 		  xvideosink->format, (gchar*)&print_format, 
 		  xvideosink->width, xvideosink->height);
 
-  g_object_freeze_notify (G_OBJECT (xvideosink));
-  g_object_notify (G_OBJECT (xvideosink), "width");
-  g_object_notify (G_OBJECT (xvideosink), "height");
-  g_object_notify (G_OBJECT (xvideosink), "format");
-  g_object_thaw_notify (G_OBJECT (xvideosink));
-
   g_mutex_lock (xvideosink->lock);
 
   /* if we have any old image settings lying around, clean that up first */
@@ -458,8 +442,6 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
 
   xvideosink->image = NULL;
   
-  GST_ELEMENT (xvideosink)->setclockfunc = gst_xvideosink_set_clock;
-
   GST_FLAG_SET(xvideosink, GST_ELEMENT_THREAD_SUGGESTED);
 }
 
@@ -487,7 +469,8 @@ gst_xvideosink_dispose (GObject *object)
   g_mutex_free (xvideosink->lock);
   g_mutex_free (xvideosink->pool_lock);
 
-  gst_buffer_pool_free (xvideosink->bufferpool);
+  if (xvideosink->bufferpool) 
+    gst_buffer_pool_free (xvideosink->bufferpool);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -592,16 +575,6 @@ gst_xvideosink_set_property (GObject *object, guint prop_id,
   xvideosink = GST_XVIDEOSINK (object);
 
   switch (prop_id) {
-    case ARG_WIDTH:
-      xvideosink->width = g_value_get_int (value);
-      xvideosink->auto_size = FALSE;
-      GST_DEBUG (0,"xvideosink: setting width to %d", xvideosink->width);
-      break;
-    case ARG_HEIGHT:
-      xvideosink->height = g_value_get_int (value);
-      xvideosink->auto_size = FALSE;
-      GST_DEBUG (0,"xvideosink: setting height to %d", xvideosink->height);
-      break;
     case ARG_FRAMES_DISPLAYED:
       xvideosink->frames_displayed = g_value_get_int (value);
       break;
@@ -662,24 +635,6 @@ gst_xvideosink_get_property (GObject *object, guint prop_id, GValue *value, GPar
   xvideosink = GST_XVIDEOSINK(object);
 
   switch (prop_id) {
-    case ARG_WIDTH: {
-      g_value_set_int (value, xvideosink->width);
-      break;
-    }
-    case ARG_HEIGHT: {
-      g_value_set_int (value, xvideosink->height);
-      break;
-    }
-    case ARG_FORMAT: {
-      gchar *fourcc_str;
-      guint32 fourcc;
-
-      fourcc = GULONG_FROM_LE (xvideosink->format);
-      fourcc_str = g_strdup_printf ("%4.4s", (gchar *)&fourcc);
-      g_value_set_string (value, fourcc_str);
-      g_free (fourcc_str);
-      break;
-    }
     case ARG_XID: {
       if (xvideosink->window)
         g_value_set_int (value, GST_XWINDOW_XID (xvideosink->window));
